@@ -8,7 +8,6 @@ import (
 	"log"
 	"os"
 	"os/exec"
-	"runtime"
 	"strconv"
 	"strings"
 	"time"
@@ -82,7 +81,7 @@ type DockerizedInstanceOptions struct {
 
 	// Options that only affect new container creation:
 	DataBindMount       string // Host path to bind-mount as /var/lib/mysql in container
-	DataTmpfs           bool   // Use tmpfs for /var/lib/mysql. Only used if no DataBindMount, runtime.GOOS is Linux, and image is from a top-level repo (e.g. "foo" but not "foo/bar")
+	DataTmpfs           bool   // Use tmpfs for /var/lib/mysql. Only used if no DataBindMount, and image is from a top-level repo (e.g. "foo" but not "foo/bar")
 	EnableBinlog        bool   // Enable or disable binary log in database server
 	LowerCaseTableNames uint8  // lower_case_table_names setting (0, 1, or 2) in database server
 }
@@ -123,11 +122,10 @@ func CreateDockerizedInstance(opts DockerizedInstanceOptions) (*DockerizedInstan
 	}
 	if opts.DataBindMount != "" {
 		dflags = append(dflags, "-v {DATABINDMOUNT}")
-	} else if opts.DataTmpfs && runtime.GOOS == "linux" && !strings.ContainsRune(opts.Image, '/') {
-		// tmpfs can only be used on *native* Linux (not Docker Desktop for Mac/Win).
-		// It can also cause permission issues with some non-Docker-official images,
-		// such as percona/percona-server; for this reason we only enable it for
-		// images from the top-level namespace, since these are known to support it.
+	} else if opts.DataTmpfs && (strings.HasPrefix(opts.Image, "mysql:") || strings.HasPrefix(opts.Image, "mariadb:")) {
+		// tmpfs cannot be used with some images, such as Percona Server. For this
+		// reason we only enable tmpfs for mysql and mariadb images from the top-level
+		// (Docker Inc maintained) namespace, since these are known to support it.
 		dflags = append(dflags, "--tmpfs /var/lib/mysql")
 	}
 	flagString := strings.Join(dflags, " ")
@@ -136,12 +134,13 @@ func CreateDockerizedInstance(opts DockerizedInstanceOptions) (*DockerizedInstan
 	// instances used only for schema management, we can configure the server in a
 	// way that reduces resource usage and improves performance for this workload
 	serverArgs := []string{
-		"--innodb-log-file-size=16777216",       // use smaller 16MB redo log files, instead of default of 48MB-96MB (varies by flavor)
+		"--innodb-log-file-size=4194304",        // use smaller 4MB redo log files, instead of default of 48MB-96MB (varies by flavor)
 		"--innodb-buffer-pool-size=33554432",    // use smaller 32MB buffer pool, instead of default of 128MB
 		"--performance-schema=0",                // disable performance_schema to reduce memory usage and other overhead
 		"--skip-innodb-adaptive-hash-index",     // AHI not beneficial to DDL-based workload
 		"--loose-innodb-log-writer-threads=off", // log writer threads harm workspace perf (loose- prefix since only in MySQL 8.0.22+)
 		"--loose-query-cache-size=0",            // ensure query cache completely disabled (loose- prefix since no longer in MySQL 8+)
+		"--skip-innodb-doublewrite",             // not needed for an ephemeral DB; perf impact for data dictionary in MySQL 8.0+
 	}
 	if opts.EnableBinlog {
 		serverArgs = append(serverArgs, "--log-bin", "--server-id=1")
@@ -186,7 +185,7 @@ func GetDockerizedInstance(opts DockerizedInstanceOptions) (*DockerizedInstance,
 		return nil, errors.New("GetDockerizedInstance: Name field cannot be empty string")
 	}
 
-	if err := startDockerContainer(opts.Name); err != nil {
+	if err := StartDockerContainer(opts.Name); err != nil {
 		return nil, err
 	}
 	di, err := newDockerizedInstance(opts)
@@ -240,6 +239,11 @@ func newDockerizedInstance(opts DockerizedInstanceOptions) (*DockerizedInstance,
 		}
 		return nil, err
 	}
+
+	// Attempt to disable redo logging, if supported by flavor. Error return of
+	// this call is intentionally ignored, since it isn't essential.
+	di.SetRedoLog(false)
+
 	return di, nil
 }
 
@@ -322,7 +326,7 @@ func GetOrCreateDockerizedInstance(opts DockerizedInstanceOptions) (*DockerizedI
 // already running, an error will be returned if it cannot be started. If it is
 // already running, nil will be returned.
 func (di *DockerizedInstance) Start() error {
-	if err := startDockerContainer(di.containerName); err != nil {
+	if err := StartDockerContainer(di.containerName); err != nil {
 		return err
 	}
 	// Randomly-assigned port on host side will have changed
@@ -336,7 +340,11 @@ func (di *DockerizedInstance) Start() error {
 	return nil
 }
 
-func startDockerContainer(name string) error {
+// StartDockerContainer shells out to `docker start` for the supplied container
+// name. If a non-zero exit code is returned, any STDOUT and STDERR output will
+// be captured and included in the returned error's message. If the container is
+// already running, nil is returned.
+func StartDockerContainer(name string) error {
 	vars := map[string]string{
 		"NAME": name,
 	}
@@ -354,8 +362,16 @@ func startDockerContainer(name string) error {
 func (di *DockerizedInstance) Stop() error {
 	di.CloseAll()
 	di.portMap = nil
+	return StopDockerContainer(di.containerName)
+}
+
+// StopDockerContainer shells out to `docker stop` for the supplied container
+// name. If a non-zero exit code is returned, any STDOUT and STDERR output will
+// be captured and included in the returned error's message. If the container
+// was already stopped, nil is returned.
+func StopDockerContainer(name string) error {
 	vars := map[string]string{
-		"NAME": di.containerName,
+		"NAME": name,
 	}
 	s := shellout.New("docker stop {NAME}").WithVariablesStrict(vars)
 	out, err := s.RunCaptureCombined()
@@ -535,12 +551,47 @@ func (di *DockerizedInstance) Exec(cmd []string, stdin io.Reader) (stdoutStr str
 	return s.RunCaptureSeparate()
 }
 
+// SetRedoLog attempts to enable or disable redo logging on the instance. This
+// only works in MySQL 8.0.21+, and otherwise will return an error. Disabling
+// the redo log improves performance, and is generally fine for the ephemeral
+// use-cases that DockerizedInstance is intended for. However, any unexpected
+// server halt/crash will render the container unable to be usable, so this
+// should be avoided for containers that are left running.
+func (di *DockerizedInstance) SetRedoLog(enable bool) error {
+	if !di.Flavor().MinMySQL(8, 0, 21) {
+		return fmt.Errorf("Cannot manipulate redo log for container %s with flavor %s", di.containerName, di.Flavor())
+	}
+	db, err := di.Instance.CachedConnectionPool("", "")
+	if err != nil {
+		return err
+	}
+	var verb string
+	if enable {
+		verb = "ENABLE"
+	} else {
+		verb = "DISABLE"
+	}
+	_, err = db.Exec("ALTER INSTANCE " + verb + " INNODB REDO_LOG")
+	return err
+}
+
 // simplifiedImageName attempts to convert the supplied image:tag string into
-// one that can be processed by ParseFlavor. It is primarily designed to convert
-// "mysql/mysql-server:tag" images into "mysql:tag" strings, and likewise for
-// "container-registry.oracle.com/mysql/community-server:tag" images.
+// one that can be processed by ParseFlavor. It is designed to simplify image
+// names that have a non-top-level namespace, e.g. "percona/percona-server",
+// "mysql/mysql-server", "container-registry.oracle.com/mysql/community-server",
+// etc into "percona" or "mysql" accordingly.
 func simplifiedImageName(image string) string {
-	base, tag, hasTag := strings.Cut(image, ":")
+	base, tag, _ := strings.Cut(image, ":")
+	tag, modifier, _ := strings.Cut(tag, "-")
+	if base == "percona/percona-server" && modifier == "aarch64" {
+		// Common special case: Percona Server images on arm always need a patch
+		// release specified, but we can strip it if it's latest
+		if tag == LatestPercona80Version.String() {
+			return "percona:8.0"
+		} else if tag == LatestPercona84Version.String() {
+			return "percona:8.4"
+		}
+	}
 	if base != "mysql" && base != "percona" && base != "mariadb" {
 		if strings.Contains(base, "maria") {
 			base = "mariadb"
@@ -550,14 +601,13 @@ func simplifiedImageName(image string) string {
 			base = "mysql"
 		}
 	}
-	if hasTag {
-		tag, _, _ = strings.Cut(tag, "-") // discard any suffix like "-aarch64"
-		if strings.Count(tag, ".") > 1 && strings.HasSuffix(tag, ".0") {
-			tag = tag[0 : len(tag)-2] // discard any ".0" point release, e.g. "8.1.0" becomes "8.1"
-		}
-		return base + ":" + tag
+	if tag == "" {
+		return base
 	}
-	return base
+	if strings.Count(tag, ".") > 1 && strings.HasSuffix(tag, ".0") {
+		tag = tag[0 : len(tag)-2] // discard any ".0" point release, e.g. "8.1.0" becomes "8.1"
+	}
+	return base + ":" + tag
 }
 
 // ContainerNameForImage returns a usable container name (or portion of a name)
