@@ -6,6 +6,7 @@ import (
 	"io"
 	"os"
 	"reflect"
+	"runtime/debug"
 	"strings"
 	"testing"
 
@@ -71,14 +72,20 @@ func RunSuite(suite IntegrationTestSuite, t *testing.T, backends []string) {
 						log.SetOutput(w)
 						outChan := make(chan []byte)
 						defer func() {
+							iface := recover()
 							w.Close()
 							os.Stdout = realOut
 							os.Stderr = realErr
 							log.SetOutput(realLogOutput)
 							testOutput := <-outChan
-							if subt.Failed() || subt.Skipped() {
+							if subt.Failed() || subt.Skipped() || iface != nil {
 								os.Stderr.Write(testOutput)
 							}
+							if iface != nil {
+								os.Stderr.WriteString(fmt.Sprintf("panic: %v [recovered]\n\n", iface))
+								os.Stderr.Write(debug.Stack())
+								subt.Fail()
+							}
 						}()
 						go func() {
 							var b bytes.Buffer
@@ -130,20 +137,50 @@ func SkeemaTestImages(t *testing.T) []string {
 
 	images := strings.Split(envString, ",")
 	for n, image := range images {
-		// No MySQL 5.x or Percona Server 5.x builds available for arm64; some
-		// Percona Server 8.0+ builds available from alternative location
+		// No MySQL 5.x or Percona Server 5.x builds available for arm64. Some
+		// Percona Server 8.0+ builds available from alternative location, but require
+		// specifying an exact patch release.
 		if arch == "arm64" {
 			if strings.HasPrefix(image, "percona:5") || strings.HasPrefix(image, "mysql:5") {
 				t.Fatalf("SKEEMA_TEST_IMAGES env var includes %s, but this image is not available for %s", image, arch)
-			} else if strings.HasPrefix(image, "percona") {
-				if strings.Count(image, ".") < 2 {
-					t.Fatalf("SKEEMA_TEST_IMAGES env var includes %s, but you MUST specify a specific patch version for Percona Server on %s", image, arch)
-				}
-				images[n] = strings.Replace(image, "percona:", "percona/percona-server:", 1) + "-aarch64"
+			} else if image == "percona:8.0" {
+				images[n] = "percona/percona-server:" + LatestPercona80Version.String() + "-aarch64"
+			} else if image == "percona:8.4" {
+				images[n] = "percona/percona-server:" + LatestPercona84Version.String() + "-aarch64"
+			} else if strings.HasPrefix(image, "percona:8.") && strings.Count(image, ".") < 2 { // 8.1-8.3 innovation releases: always .0 patch
+				images[n] = strings.Replace(image, "percona:", "percona/percona-server:", 1) + ".0-aarch64"
 			}
-		} else if strings.HasPrefix(image, "percona") && ParseFlavor(image).MinMySQL(8, 1) {
+		} else if strings.HasPrefix(image, "percona:8") {
+			// Top-level (Docker Inc maintained) images for Percona Server 8.0 appear to
+			// not be updated frequently anymore, so always use percona/percona-server
+			// now even on Intel/AMD. Tags for 8.x exist for Intel/AMD without having to
+			// force a patch version number.
 			images[n] = strings.Replace(image, "percona:", "percona/percona-server:", 1)
 		}
 	}
 	return images
 }
+
+// SkeemaTestContainerCleanup potentially performs cleanup on a container used
+// in integration testing, depending on the value of the SKEEMA_TEST_CLEANUP
+// env variable:
+//   - If SKEEMA_TEST_CLEANUP is set to "stop" (case-insensitive), the supplied
+//     container will be stopped.
+//   - If SKEEMA_TEST_CLEANUP is set to "none" (case-insensitive), no cleanup
+//     action is taken, and the supplied container remains running.
+//   - Otherwise (if SKEEMA_TEST_CLEANUP is not set, or set to any other value),
+//     the supplied container will be removed (destroyed) if it has a name
+//     beginning with "skeema-test-". With any other name prefix, an error is
+//     returned.
+func SkeemaTestContainerCleanup(d *DockerizedInstance) error {
+	action := strings.TrimSpace(os.Getenv("SKEEMA_TEST_CLEANUP"))
+	if strings.EqualFold(action, "none") {
+		return nil
+	} else if strings.EqualFold(action, "stop") {
+		return d.Stop()
+	} else if strings.HasPrefix(d.containerName, "skeema-test-") {
+		return d.Destroy()
+	} else {
+		return fmt.Errorf("refusing to destroy container %q without skeema-test- naming prefix", d.containerName)
+	}
+}
